'use strict';

var babelCore = require('babel-core');
var RawModule = require('webpack/lib/RawModule');
var SingleEntryDependencyWrapper = require('../SingleEntryDependencyWrapper');
var _Symbol = require('symbol');
var NullFactory = require('webpack/lib/NullFactory');
var pluginOptionsController = require('../../plugin/options');

var moduleIdentifierSymbol = _Symbol('topLevelModuleIdentifier');

module.exports = function (compiler) {
    compiler.plugin('compilation', handleCompilation);
};

function handleCompilation(compilation) {
    compilation.dependencyFactories.set(SingleEntryDependencyWrapper, new NullFactory());
    compilation.plugin('seal', modifyChunks);
}

function modifyChunks() {
    var _this = this;

    getChunks(this).forEach(function (chunk) {
        return injectHelpers(_this, chunk.module);
    });
}

function getChunks(compilation) {
    var addToSpecificEntries = pluginOptionsController.get('entries');
    var chunks = compilation.preparedChunks;
    if (addToSpecificEntries.length) {
        return chunks.filter(function (chunk) {
            return addToSpecificEntries.indexOf(chunk.name) > -1;
        });
    }

    return chunks;
}

function injectHelpers(compilation, topLevelModule) {
    var dependency = getDependency(topLevelModule);
    var module = createModule(dependency);
    var cachedModule = compilation.addModule(module);
    if (cachedModule instanceof RawModule) {
        module = cachedModule;
    }
    linkModules(topLevelModule, module, dependency);
    if (cachedModule === true) {
        topLevelModule.dependencies.unshift(dependency);
        compilation.buildModule(module, function (error) {
            if (error) {
                throw new Error('Helpers module is not build.');
            }
        });
    }
}

function getDependency(topLevelModule) {
    return findInjectedDependency(topLevelModule) || createDependency(topLevelModule);
}

function findInjectedDependency(topLevelModule) {
    return topLevelModule.dependencies.filter(function (dependency) {
        return dependency instanceof SingleEntryDependencyWrapper;
    })[0];
}

function createDependency(topLevelModule) {
    var dependency = new SingleEntryDependencyWrapper('babel-loader-external-helpers');
    dependency.loc = topLevelModule.name + ":" + (100000 - 1);
    dependency[moduleIdentifierSymbol] = topLevelModule.identifier();

    return dependency;
}

function createModule(dependency) {
    return new RawModule(buildBabelHelpers(), getModuleIdentifier(dependency));
}

function buildBabelHelpers() {
    var whitelistArgument = void 0;
    var whitelist = pluginOptionsController.get('whitelist');
    if (whitelist.length) {
        whitelistArgument = whitelist;
    }

    return babelCore.buildExternalHelpers(whitelistArgument);
}

function getModuleIdentifier(dependency) {
    return 'generated babel-helpers ' + dependency[moduleIdentifierSymbol];
}

function linkModules(topLevelModule, module, dependecy) {
    module.issuer = topLevelModule.identifier();
    dependecy.module = module;
    module.addReason(topLevelModule, dependecy);
}